-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

--round from http://lua-users.org/wiki/SimpleRound


local M = {}

local fuelCapacity, speed, rpm, gear, fuel, water, boost, pNightMode
local maxRpm, rpmC, rpmUp, rpmDn, rpmP
local wasteStart, wasteLimit, boost, boostP, curWheel

local unitVel = 3.6
local unitVol = 1
local unitTemp = 1
local unitTemp1 = 0

local brakeFL, brakeFR, brakeRL, brakeRR

local vehicleID = obj:getID()
local dashScreenName = "@etki_screen"
local nightMode = 0


local function round(num, idp)
	if num then
		local mult = 10^(idp or 0)
		return math.floor(num * mult + 0.5) / mult
	else
		return 0
	end
end



local function runFunc(func)
	obj:queueGameEngineLua("if be:getObjectByID("..vehicleID..") then be:getObjectByID("..vehicleID.."):queueJSUITexture('"..dashScreenName.."', '"..func.."') end")
end





local function onInit()
	if v.data.mainEngine.digital_gauges then
		obj:queueGameEngineLua("extensions.loadModule('ui/uinavi')")
		obj:queueGameEngineLua('be:getObjectByID('..vehicleID..'):createUITexture("'..dashScreenName..'", "local://local/vehicles/etki/rsx_dash.html", 1024, 512, UI_TEXTURE_USAGE_AUTOMATIC, 30)')
		fuelCapacity = v.data.mainTank.fuelCapacity
		
		maxRpm = v.data.mainEngine.maxRPM
		rpmUp = v.data.vehicleController.highShiftUpRPM
		rpmDn = 0
		--rpmDn = v.data.mainEngine.shiftDownRPM
		
		electrics.values.nightmode = 0
		
		if v.data.turbocharger and wasteStart and wasteLimit then
			wasteStart = v.data.turbocharger.wastegateStart
			wasteLimit = v.data.turbocharger.wastegateLimit
		else
			wasteStart = 1
			wasteLimit = 1
		end
		
		
		if v.data.mainEngine.digital_gauges == "imperial" then
			unitVel = 2.237
			unitVol = 0.264
			unitTemp = 1.8
			unitTemp1 = 32
			
			runFunc("imperial()")
		end
		
	else
		return
	end
end

local function onReset()
    onInit()
	if electrics.values["nightmode"] == 0 then
		runFunc("nightMode(0)")
	else
		runFunc("nightMode(1)")
	end
end

local function updateGFX(dt)
	if v.data.mainEngine.digital_gauges then
		
		speed = math.floor(((electrics.values.wheelspeed or 0) * unitVel) + 0.5)
		rpm = math.floor(electrics.values.rpm + 0.5)
		
		rpmP = rpm / maxRpm
		
		if rpm < rpmDn then rpmC = -1 elseif rpm > rpmUp then rpmC = 1 else rpmC = 0 end
		
		gear = math.floor(electrics.values.gear + 0.5)
		fuel = math.floor(electrics.values.fuel * fuelCapacity * unitVol + 0.5)
		water = math.floor((electrics.values.watertemp * unitTemp + unitTemp1) + 0.5)
		
		
		
		boost = electrics.values.turboBoost
		if boost == nil or boost < 0 then boost = 0 end
		
		
		boostP = boost / ((wasteStart + wasteLimit) / 2.16)
		if boostP > 1 then boostP = 1 end
		
		
		
		for i=0, 3 do
			curWheel = wheels.wheels[i]
			if curWheel.name == "FL" then
				brakeFL = math.floor((curWheel.brakeCoreTemperature * unitTemp + unitTemp1) + 0.5)
			elseif curWheel.name == "FR" then
				brakeFR = math.floor((curWheel.brakeCoreTemperature * unitTemp + unitTemp1) + 0.5)
			elseif curWheel.name == "RL" then
				brakeRL = math.floor((curWheel.brakeCoreTemperature * unitTemp + unitTemp1) + 0.5)
			else
				brakeRR = math.floor((curWheel.brakeCoreTemperature * unitTemp + unitTemp1) + 0.5)
			end
		end
	
		pNightMode = nightMode
		nightMode = electrics.values["nightmode"]
		--print(electrics.values.nightmode)
		
		if nightMode == 1 then
			if pNightMode == 0 then runFunc("nightMode(1)") end
			runFunc("updateSpeed("..speed..")")
			runFunc("updateGear("..gear..")")
			runFunc("updateRpm("..rpm..", 0 ,"..rpmC..")")
			runFunc("updateFuel("..fuel..")")
			runFunc("updateWater("..water..")")
		else
			if pNightMode == 1 then runFunc("nightMode(0)") end
			runFunc("updateSpeed("..speed..")")
			runFunc("updateGear("..gear..")")
			runFunc("updateRpm("..rpm..","..rpmP..","..rpmC..")")
			runFunc("updateBrakes("..brakeFL..","..brakeFR..","..brakeRL..","..brakeRR..")")
			runFunc("updateFuel("..fuel..")")
			runFunc("updateWater("..water..")")
			runFunc("updateBoost("..round(boost * 0.0689476, 1)..","..boostP..")")
		end
		
	else
		return
	end
	
	
end

-- public interface
M.onInit    = onInit
M.onReset   = onReset
M.updateGFX = updateGFX

return M
